// First element.
Array.prototype.head = function ()
{
	return this[0];
};

// Search using a predicate
//
//	[1, 2, 3].search (function (i) { return i > 2; })
//		== 2
Array.prototype.search = function (p)
{
	for (var i = 0; i < this.length; ++i)
		if (p (this[i])) return i;
	return -1;
};

// Last element.
Array.prototype.last = function ()
{
	return this[-1];
};

// All but first element.
Array.prototype.tail = function ()
{
	return this.slice (1);
};

// Predicate p is true for all elements.
Array.prototype.all = function (p)
{
	for (var i = 0; i < this.length; ++i)
		if (!p (this[i]))
			return false;
	return true;
};

// Predicate p is true for some elemement.
Array.prototype.any = function (p)
{
	for (var i = 0; i < this.length; ++i)
		if (p (this[i]))
			return true;
	return false;
};

// Collect elements for which predicate p is true.
Array.prototype.filter = function (p)
{
	var xs = [];
	for (var i = 0; i < this.length; ++i)
		if (p (this[i]))
			xs.push (this[i]);
	return xs;
};

// Return copy of self, in which f has been applied to each element.
Array.prototype.map = function (f)
{
	var f_this = this.slice ();
	for (var i = 0; i < this.length; ++i)
		f_this[i] = f (this[i]);
	return f_this;
};

Array.prototype.concatAll = function ()
{
  if (typeof this[0] == 'string')
    return this.reduce ("", function (a, b) { return a + b; });
  else
    return this.reduce ([], function (a, b) { return a.concat (b); });
};

// Perform function f on each element, returning this.
Array.prototype.each = function (f)
{
	for (var i = 0; i < this.length; ++i)
		f (this[i]);
	return this;
};

// Reduce array into accumulator with function f.
Array.prototype.reduce = function (acc, f)
{
	for (var i = 0; i < this.length; ++i) {
		var acc2 = f (acc, this[i]);
		// We just reuse the previous accumulator if f does not return a value. If
		// the accumulator is an object, this obviates the need to return a
		// reference to the object in the passed function.
		acc = acc2 != undefined ? acc2 : acc;
	}
	return acc;
};

// Join an array of strings into a newline-separated string.
Array.prototype.unlines = function ()
{
	return this.drop (1).reduce (this[0], function (ss, s) { return ss+"\n"+s; });
};

// Join an array of strings into a space-separated string.
Array.prototype.unwords = function ()
{
	return this.drop (1).reduce (this[0], function (ss, s) { return ss+" "+s; });
};

// Repeat the array n times.
Array.prototype.repeat = function (n)
{
	if (n <= 1) return this;
	return this.concat (this.repeat (n - 1));
};

Array.prototype.drop = function (n)
{
	return this.slice (Math.min (n, this.length));
};

Array.prototype.take = function (n)
{
	return this.slice (0, Math.min (n, this.length));
};

// Return a random element from the array.
Array.prototype.random = function ()
{
  return this[Math.floor (Math.random () * this.length)];
};

// All elements in array are true.
Array.prototype.and = function ()
{
	for (var i = 0; i < this.length; ++i)
		if (!this[i]) return false;
	return true;
};

// Any element in array is true.
Array.prototype.or = function ()
{
	for (var i = 0; i < this.length; ++i)
		if (this[i]) return true;
	return false;
};

